"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const compare = require("reg-cli");
const rimraf = require("rimraf");
const cpx = require("cpx");
class RegProcessor {
    constructor(opt) {
        this._logger = opt.logger;
        this._config = opt.coreConfig;
        this._directoryInfo = {
            workingDirs: opt.workingDirs,
            userDirs: opt.options.userDirs,
        };
        this._keyGenerator = opt.options.keyGenerator;
        this._publisher = opt.options.publisher;
        this._notifiers = opt.options.notifiers;
    }
    runAll() {
        return this.getExpectedKey()
            .then(ctx => this.syncExpected(ctx))
            .then(ctx => this.compare(ctx))
            .then(ctx => this.getActualKey(ctx))
            .then(ctx => this.publish(ctx))
            .then(ctx => this.notify(ctx));
    }
    getExpectedKey() {
        if (this._keyGenerator) {
            return this._keyGenerator.getExpectedKey()
                .then(key => {
                this._logger.info(`Detected the previous snapshot key: '${key}'`);
                return { expectedKey: key };
            })
                .catch(reason => {
                this._logger.warn("Failed to detect the previous snapshot key");
                this._logger.error(reason);
                return Promise.resolve({ expectedKey: null });
            });
        }
        else {
            this._logger.info("Skipped to detect the previous snapshot key because key generator plugin is not set up.");
            return Promise.resolve({ expectedKey: null });
        }
    }
    compare(ctx) {
        const { actualDir, expectedDir, diffDir } = this._directoryInfo.workingDirs;
        const json = path.join(this._directoryInfo.workingDirs.base, "out.json");
        const report = path.join(this._directoryInfo.workingDirs.base, "index.html");
        rimraf.sync(actualDir);
        cpx.copySync(`${this._directoryInfo.userDirs.actualDir}/**/*.{png,jpg,jpeg,tiff,bmp,gif}`, actualDir);
        return compare({
            actualDir,
            expectedDir,
            diffDir,
            json,
            report,
            update: false,
            ignoreChange: true,
            urlPrefix: "",
            threshold: this._config.threshold,
        })
            .then(result => {
            this._logger.verbose("Comparison result:", result);
            return Object.assign({}, ctx, { comparisonResult: result });
        })
            .catch(reason => {
            // re-throw notifiers error because it's fatal.
            this._logger.error("An error occurs during compare images:");
            this._logger.error(reason);
            return Promise.reject(reason);
        });
    }
    getActualKey(ctx) {
        const fallbackFn = () => "snapshot_" + ~~(new Date().getTime() / 1000);
        if (this._keyGenerator) {
            return this._keyGenerator.getActualKey().then(key => {
                if (!key) {
                    this._logger.warn("Failed to generate the current snapshot key.");
                    return Object.assign({}, ctx, { actualKey: fallbackFn() });
                }
                this._logger.info(`The current snapshot key: '${key}'`);
                return Object.assign({}, ctx, { actualKey: key });
            }).catch(reason => {
                this._logger.warn("Failed to gerenate the current snapshot key.");
                this._logger.error(reason);
                return Promise.resolve(Object.assign({}, ctx, { actualKey: fallbackFn() }));
            });
        }
        else {
            const fallbackKey = fallbackFn();
            this._logger.info(`Use '${fallbackKey}' as the current snapshot key because key generator plugin is not set up.`);
            return Promise.resolve(Object.assign({}, ctx, { actualKey: fallbackKey }));
        }
    }
    syncExpected(ctx) {
        const keyForExpected = ctx.expectedKey;
        if (this._publisher && keyForExpected) {
            return this._publisher.fetch(keyForExpected);
        }
        else if (!keyForExpected) {
            this._logger.info("Skipped to fetch the expeceted data because expected key is null.");
            return Promise.resolve(ctx);
        }
        else if (!this._publisher) {
            this._logger.info("Skipped to fetch the expeceted data because publisher plugin is not set up.");
            return Promise.resolve(ctx);
        }
        else {
            return Promise.resolve(ctx);
        }
    }
    publish(ctx) {
        if (this._publisher) {
            return this._publisher.publish(ctx.actualKey)
                .then(result => {
                this._logger.info(`Published snapshot '${ctx.actualKey}' successfully.`);
                if (result.reportUrl) {
                    this._logger.info(`Report URL: ${result.reportUrl}`);
                }
                this._logger.verbose("Publish result:", result);
                return Object.assign({}, ctx, { reportUrl: result.reportUrl });
            })
                .catch(reason => {
                // re-throw notifiers error because it's fatal.
                this._logger.error("An error occurs during publishing snapshot:");
                this._logger.error(reason);
                return Promise.reject(reason);
            });
        }
        else {
            this._logger.info("Skipped to publish the snapshot data because publisher plugin is not set up.");
            return Promise.resolve(ctx);
        }
    }
    notify(ctx) {
        const notifyParams = Object.assign({}, ctx);
        if (!this._notifiers.length) {
            this._logger.info("Skipped to notify result because notifier plugins are not set up.");
        }
        this._logger.verbose("Notify parameters:", notifyParams);
        return this._notifiers.reduce((queue, notifier) => {
            return queue
                .then(() => notifier.notify(notifyParams))
                .catch((reason) => {
                // Don't re-throw notifiers error because it's not fatal.
                this._logger.error("An error occurs during notify:");
                this._logger.error(reason);
                return Promise.resolve();
            });
        }, Promise.resolve()).then(() => ctx);
    }
}
exports.RegProcessor = RegProcessor;
//# sourceMappingURL=processor.js.map